# coding=utf-8
import logging
import sys
import traceback

from django.conf import settings
from django.contrib.auth.models import User

from .subject_import_reader import SubjectImportReader
from .warning_counter import MsgCounterHandler
from web.models import StudySubject, Subject, Provenance, Worker
from web.models.constants import GLOBAL_STUDY_ID

logger = logging.getLogger(__name__)


class Importer(object):
    def __init__(self, filename, reader):
        # type: (str, SubjectImportReader) -> None
        self.filename = filename
        self.reader = reader
        self.added_count = 0
        self.problematic_count = 0
        self.merged_count = 0
        self.warning_count = 0
        self.study_subjects = []

        self.importer_user = None

        importer_user_name = getattr(settings, "IMPORTER_USER", None)
        if importer_user_name is not None:
            user = User.objects.filter(username=importer_user_name)
            if user is None:
                logger.warn("User does not exist: " + importer_user_name)
            else:
                self.importer_user = Worker.objects.filter(user=user)


    def execute(self):
        self.added_count = 0
        self.problematic_count = 0
        self.merged_count = 0
        self.warning_count = 0

        warning_counter = MsgCounterHandler()
        logging.getLogger('').addHandler(warning_counter)

        self.study_subjects = self.reader.load_data(self.filename)
        for study_subject in self.study_subjects:
            try:
                if study_subject.study is None:
                    self.problematic_count += 1
                    logger.warn("Empty study found. Ignoring")
                    continue
                elif study_subject.study.id != GLOBAL_STUDY_ID:
                    self.problematic_count += 1
                    logger.warn("Empty study found. Ignoring: " + study_subject.study.id)
                    continue
                else:
                    self.import_study_subject(study_subject)
            except:
                self.problematic_count += 1
                traceback.print_exc(file=sys.stdout)
                logger.error("Problem with importing study subject: " + study_subject.screening_number)
        if "WARNING" in warning_counter.level2count:
            self.warning_count = warning_counter.level2count["WARNING"]
        logging.getLogger('').removeHandler(warning_counter)

    def import_study_subject(self, study_subject):
        # type: (StudySubject) -> None
        db_study_subjects = StudySubject.objects.filter(nd_number=study_subject.nd_number)
        if db_study_subjects.count() > 0:
            db_study_subject = db_study_subjects.first()
            for field in Subject._meta.get_fields():
                if field.get_internal_type() == "CharField" or field.get_internal_type() == "DateField" or field.get_internal_type() is "BooleanField":
                    new_value = getattr(study_subject.subject, field.name)
                    if new_value is not None and new_value != "":
                        old_value = getattr(db_study_subject.subject, field.name)
                        description = '{} changed from "{}" to "{}"'.format(field, old_value, new_value)
                        p = Provenance(modified_table=Subject._meta.db_table,
                                       modified_table_id=db_study_subject.subject.id,
                                       modification_author=self.importer_user,
                                       previous_value=old_value,
                                       new_value=new_value,
                                       modification_description=description,
                                       modified_field=field,
                                       )
                        setattr(db_study_subject.subject, field.name, new_value)
                        p.save()

            db_study_subject.subject.save()

            for field in StudySubject._meta.get_fields():
                if field.get_internal_type() == "CharField" or field.get_internal_type() == "DateField" or field.get_internal_type() is "BooleanField":
                    new_value = getattr(study_subject, field.name)
                    if new_value is not None and new_value != "":
                        old_value = getattr(db_study_subject, field.name)
                        description = '{} changed from "{}" to "{}"'.format(field, old_value, new_value)
                        p = Provenance(modified_table=Subject._meta.db_table,
                                       modified_table_id=db_study_subject.id,
                                       modification_author=self.importer_user,
                                       previous_value=old_value,
                                       new_value=new_value,
                                       modification_description=description,
                                       modified_field=field,
                                       )
                        setattr(db_study_subject, field.name, new_value)
                        p.save()
            db_study_subject.save()
            self.merged_count += 1
        else:
            study_subject.subject.save()
            study_subject.subject = Subject.objects.filter(id=study_subject.subject.id)[0]
            study_subject.save()

            for field in Subject._meta.get_fields():
                if field.get_internal_type() == "CharField" or field.get_internal_type() == "DateField" or field.get_internal_type() is "BooleanField":
                    new_value = getattr(study_subject.subject, field.name)
                    if new_value is not None and new_value != "":
                        description = '{} changed from "{}" to "{}"'.format(field, '', new_value)
                        p = Provenance(modified_table=Subject._meta.db_table,
                                       modified_table_id=study_subject.subject.id,
                                       modification_author=self.importer_user,
                                       previous_value='',
                                       new_value=new_value,
                                       modification_description=description,
                                       modified_field=field,
                                       )
                        p.save()

            for field in StudySubject._meta.get_fields():
                if field.get_internal_type() == "CharField" or field.get_internal_type() == "DateField" or field.get_internal_type() is "BooleanField":
                    new_value = getattr(study_subject, field.name)
                    if new_value is not None and new_value != "":
                        description = '{} changed from "{}" to "{}"'.format(field, '', new_value)
                        p = Provenance(modified_table=Subject._meta.db_table,
                                       modified_table_id=study_subject.id,
                                       modification_author=self.importer_user,
                                       previous_value='',
                                       new_value=new_value,
                                       modification_description=description,
                                       modified_field=field,
                                       )
                        p.save()

            self.added_count += 1

    def get_summary(self):
        result = "<p>Number of entries: <b>" + str(len(self.study_subjects)) + "</b></p>" + \
                 "<p>Number of successfully added entries: <b>" + str(self.added_count) + "</b></p>" + \
                 "<p>Number of successfully merged entries: <b>" + str(self.merged_count) + "</b></p>"
        style = ''
        if self.problematic_count > 0:
            style = ' color="red" '
        result += "<p><font " + style + ">Number of problematic entries: <b>" + str(
            self.problematic_count) + "</b></font></p>"
        style = ''
        if self.warning_count > 0:
            style = ' color="brown" '
        result += "<p><font " + style + ">Number of raised warnings: <b>" + str(self.warning_count) + "</b></font></p>"

        return result
